@@ -6,14 +6,16 @@ use 5.008_005;
 use utf8;
 use open OUT => qw< :encoding(UTF-8) :std >;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
+use Config;
 use URI::Escape qw(uri_escape);
 use HTTP::Tiny;
 use JSON qw(decode_json);
 use CPAN::DistnameInfo;
-use Term::ANSIColor qw(GREEN BLUE RED BOLD RESET);
 
+our $SERVER = "http://grep.cpan.me";
+our $COLOR;
 our $DEBUG;
 
 # TODO:
@@ -27,11 +29,17 @@ our $DEBUG;
 sub run {
     require Getopt::Long;
     Getopt::Long::GetOptions(
+        'color!'    => \$COLOR,
         'd|debug!'  => \$DEBUG,
         'h|help'    => \(my $help),
         'version'   => \(my $version),
+
+        'l'         => \(my $list),
+        'server=s'  => \$SERVER,
     );
 
+    setup_colors() unless defined $COLOR and not $COLOR;
+
     if ($help) {
         print help();
         return 0;
@@ -52,7 +60,11 @@ sub run {
         my $search = search($query)
             or return 2;
 
-        display($search);
+        if ($list) {
+            display_list($search);
+        } else {
+            display($search);
+        }
         return 0;
     }
     return 0;
@@ -69,6 +81,16 @@ See <http://grep.cpan.me/about#re> for more information.
 
 Multiple query arguments will be joined with spaces for convenience.
 
+  -l            List only matching filenames.  Note that omitted results are
+                not mentioned, but your pattern is likely to match many more
+                files than output.
+
+  --color       Enable colored output even if STDOUT isn't a terminal
+  --no-color    Disable colored output
+
+  --server      Specifies an alternate server to use, for example:
+                    --server http://localhost:5000
+
   --debug       Print debug messages to stderr
   --help        Show this help and exit
   --version     Show version
@@ -76,8 +98,8 @@ Multiple query arguments will be joined with spaces for convenience.
     USAGE
 }
 
-sub search_url     { "http://grep.cpan.me/?q="    . uri_escape(shift) }
-sub search_api_url { "http://grep.cpan.me/api?q=" . uri_escape(shift) }
+sub search_url     { "$SERVER/?q="    . uri_escape(shift) }
+sub search_api_url { "$SERVER/api?q=" . uri_escape(shift) }
 
 sub search {
     my $query = shift;
@@ -116,31 +138,82 @@ sub display {
         my $dist = CPAN::DistnameInfo->new($fulldist);
 
         for my $file (@{$result->{files}}) {
-            print GREEN, join("/", $dist->cpanid, $dist->distvname, $file->{file}), RESET, "\n";
+            print colored(["GREEN"], join("/", $dist->cpanid, $dist->distvname, $file->{file})), "\n";
 
             for my $match (@{$file->{results}}) {
                 my $snippet = $match->{text};
 
-                substr($snippet, $match->{match}[1], 0) = RESET;
-                substr($snippet, $match->{match}[0], 0) = BOLD RED;
+                my ($start, $len) = @{$match->{match}};
+                $len -= $start;
+
+                substr($snippet, $start, $len) = colored(substr($snippet, $start, $len), "BOLD RED");
+
+                if ($match->{line}) {
+                    my $ln       = $match->{line}[0] - (substr($snippet, 0, $start) =~ y/\n//);
+                    my $print_ln = sub {
+                        colored($ln++, "BLUE") . colored(":", "CYAN")
+                    };
+                    $snippet =~ s/^/$print_ln->()/mge;
+                }
 
                 chomp $snippet;
-                $snippet =~ s/^/  /mg;
-
-                print $snippet, RESET, "\n\n";
-
-                # XXX TODO: Display line numbers
+                print $snippet, color("reset"), "\n\n";
             }
-            printf "  → %d more match%s from this file.\n\n",
+            printf colored("  → %d more match%s from this file.\n\n", "MAGENTA"),
                 $file->{truncated}, ($file->{truncated} != 1 ? "es" : "")
                     if $file->{truncated};
         }
-        printf "→ %d more file%s matched in %s.\n\n",
+        printf colored("→ %d more file%s matched in %s.\n\n", "MAGENTA"),
             $result->{truncated}, ($result->{truncated} != 1 ? "s" : ""), $dist->distvname
                 if $result->{truncated};
     }
 }
 
+sub display_list {
+    my $search  = shift or return;
+    my $results = $search->{results} || [];
+    for my $result (@$results) {
+        my $fulldist = $result->{dist};
+           $fulldist =~ s{^(?=(([A-Z])[A-Z]))}{$2/$1/};
+        my $dist = CPAN::DistnameInfo->new($fulldist);
+
+        for my $file (@{$result->{files}}) {
+            print join("/", $dist->cpanid, $dist->distvname, $file->{file}), "\n";
+        }
+    }
+}
+
+# Setup colored output if we have it
+sub setup_colors {
+    eval { require Term::ANSIColor };
+    if ( not $@ and supports_color() ) {
+        $Term::ANSIColor::EACHLINE = "\n";
+        *color   = *_color_real;
+        *colored = *_colored_real;
+    }
+}
+
+# No-op passthrough defaults
+sub color           { "" }
+sub colored         { ref $_[0] ? @_[1..$#_] : $_[0] }
+sub _color_real     { Term::ANSIColor::color(@_) }
+sub _colored_real   { Term::ANSIColor::colored(@_) }
+
+sub supports_color {
+    # We're not on a TTY and don't force it, kill color
+    return 0 unless -t *STDOUT or $COLOR;
+
+    if ( $Config{'osname'} eq 'MSWin32' ) {
+        eval { require Win32::Console::ANSI; };
+        return 1 if not $@;
+    }
+    else {
+        return 1 if $ENV{'TERM'} =~ /^(xterm|rxvt|linux|ansi|screen)/;
+        return 1 if $ENV{'COLORTERM'};
+    }
+    return 0;
+}
+
 sub debug {
     return unless $DEBUG;
     warn "DEBUG: ", @_, " [", join("/", (caller(1))[3,2]), "]\n";
